@@ -38,6 +38,7 @@
 //#define SGL_DEBUG_SIMPLE
 //#define SGL_DEBUG_COMPLEX
 //#define SGL_DEBUG_INFO_ALL
+#define PRINT_BACKTRANCE
 //#define SGL_DEBUG_INFO_STEPSIZE
 #else
 // Do no debugging
@@ -45,6 +46,10 @@
 #define NDEBUG
 #endif
 
+#ifdef PRINT_BACKTRANCE
+#include <Backtrace.h>
+#endif
+
 #include <armadillo>
 #include <Rcpp.h>
 
